Package: singleCellTK
Type: Package
Title: Comprehensive and Interactive Analysis of Single Cell RNA-Seq
        Data
Version: 2.20.0
Authors@R: c(person(given="Yichen", family="Wang", email="wangych@bu.edu", role=c("aut"),
                    comment = c(ORCID = "0000-0003-4347-5199")),
             person(given="Irzam", family="Sarfraz", email="isarfraz@bu.edu", role=c("aut"),
                    comment = c(ORCID = "0000-0001-8121-792X")),
             person(given="Rui", family="Hong", email="rzhong@bu.edu", role=c("aut")),
             person(given="Yusuke", family="Koga", email="ykoga07@bu.edu", role=c("aut")),
             person(given="Salam", family="Alabdullatif", email="salamha@bu.edu", role=c("aut")),
             person(given="Nida", family="Pervaiz", email="nidapervaiz27@gmail.com", role=c("aut")),
             person(given="David", family="Jenkins", email="dfj@bu.edu", role=c("aut"),
                    comment = c(ORCID = "0000-0002-7451-4288")),
             person(given="Vidya", family="Akavoor", email="vidyaap@bu.edu", role=c("aut")),
             person(given="Xinyun", family="Cao", email="xcao19@bu.edu", role=c("aut")),
             person(given="Shruthi", family="Bandyadka", email="sbandya@bu.edu", role=c("aut")),
             person(given="Anastasia", family="Leshchyk", email="aleshchk@bu.edu", role=c("aut")),
             person(given="Tyler", family="Faits", email="tfaits@bu.edu", role=c("aut")),
             person(given=c("Mohammed", "Muzamil"), family="Khan", email="mmkhan@bu.edu", role=c("aut")),
             person(given="Zhe", family="Wang", email="zhe@bu.edu", role=c("aut")),
             person(given=c("W.", "Evan"), family="Johnson", email="wej@bu.edu", role=c("aut"),
                    comment = c(ORCID = "0000-0002-6247-6595")),
             person(given="Ming", family="Liu", email="mingl@bu.edu", role=c("aut")),
             person(given=c("Joshua", "David"), family="Campbell", email="camp@bu.edu", role=c("aut", "cre"),
             comment = c(ORCID = "0000-0003-0780-8662"))
             )
Depends: R (>= 4.0), SummarizedExperiment, SingleCellExperiment,
        DelayedArray, Biobase
Description: The Single Cell Toolkit (SCTK) in the singleCellTK package provides an interface to popular tools for importing, quality control, analysis, and visualization of single cell RNA-seq data. SCTK allows users to seamlessly integrate tools from various packages at different stages of the analysis workflow. A general "a la carte" workflow gives users the ability access to multiple methods for data importing, calculation of general QC metrics, doublet detection, ambient RNA estimation and removal, filtering, normalization, batch correction or integration, dimensionality reduction, 2-D embedding, clustering, marker detection, differential expression, cell type labeling, pathway analysis, and data exporting. Curated workflows can be used to run Seurat and Celda. Streamlined quality control can be performed on the command line using the SCTK-QC pipeline. Users can analyze their data using commands in the R console or by using an interactive Shiny Graphical User Interface (GUI). Specific analyses or entire workflows can be summarized and shared with comprehensive HTML reports generated by Rmarkdown. Additional documentation and vignettes can be found at camplab.net/sctk.
License: MIT + file LICENSE
Encoding: UTF-8
biocViews: SingleCell, GeneExpression, DifferentialExpression,
        Alignment, Clustering, ImmunoOncology, BatchEffect,
        Normalization, QualityControl, DataImport, GUI
LazyData: FALSE
Imports: ape, anndata, AnnotationHub, batchelor, BiocParallel, celldex,
        colourpicker, colorspace, cowplot, cluster, ComplexHeatmap,
        data.table, DelayedMatrixStats, DESeq2, dplyr, DT,
        ExperimentHub, ensembldb, fields, ggplot2, ggplotify, ggrepel,
        ggtree, gridExtra, grid, GSVA (>= 1.50.0), GSVAdata, igraph,
        KernSmooth, limma, MAST, Matrix (>= 1.6-1), matrixStats,
        methods, msigdbr, multtest, plotly, plyr, ROCR, Rtsne,
        S4Vectors, scater, scMerge (>= 1.2.0), scran, Seurat (>=
        3.1.3), shiny, shinyjs, SingleR, stringr, SoupX, sva, reshape2,
        shinyalert, circlize, enrichR (>= 3.2), celda, shinycssloaders,
        DropletUtils, scds (>= 1.2.0), reticulate (>= 1.14), tools,
        tximport, tidyr, eds, withr, GSEABase, R.utils, zinbwave,
        scRNAseq (>= 2.0.2), TENxPBMCData, yaml, rmarkdown, magrittr,
        scDblFinder, metap, VAM (>= 0.5.3), tibble, rlang, TSCAN,
        TrajectoryUtils, scuttle, utils, stats, zellkonverter,
        lifecycle
RoxygenNote: 7.3.3
Suggests: testthat, Rsubread, BiocStyle, knitr, lintr, spelling,
        org.Mm.eg.db, kableExtra, shinythemes, shinyBS, shinyjqui,
        shinyWidgets, shinyFiles, BiocGenerics, RColorBrewer, fastmap
        (>= 1.1.0), harmony, SeuratObject, optparse
VignetteBuilder: knitr
URL: https://www.camplab.net/sctk/
BugReports: https://github.com/compbiomed/singleCellTK/issues
Language: en-US
git_url: https://git.bioconductor.org/packages/singleCellTK
git_branch: RELEASE_3_22
git_last_commit: e2bff7b
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:33:33 UTC; biocbuild
Author: Yichen Wang [aut] (ORCID: <https://orcid.org/0000-0003-4347-5199>),
  Irzam Sarfraz [aut] (ORCID: <https://orcid.org/0000-0001-8121-792X>),
  Rui Hong [aut],
  Yusuke Koga [aut],
  Salam Alabdullatif [aut],
  Nida Pervaiz [aut],
  David Jenkins [aut] (ORCID: <https://orcid.org/0000-0002-7451-4288>),
  Vidya Akavoor [aut],
  Xinyun Cao [aut],
  Shruthi Bandyadka [aut],
  Anastasia Leshchyk [aut],
  Tyler Faits [aut],
  Mohammed Muzamil Khan [aut],
  Zhe Wang [aut],
  W. Evan Johnson [aut] (ORCID: <https://orcid.org/0000-0002-6247-6595>),
  Ming Liu [aut],
  Joshua David Campbell [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0780-8662>)
Maintainer: Joshua David Campbell <camp@bu.edu>
