# description
library(singleCellTK)
test_that(desc = "Testing description", {
    a <- singleCellTK:::description_subsetSCECols()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionBarcodeRank()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionBCDS()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionCXDS()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionDecontX()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionDoubletFinder()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionEmptyDrops()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionRunCellQC()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionRunDropletQC()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionRunPerCellQC()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionScDblFinder()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionScdsHybrid()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionScrublet()
    testthat::expect_type(a, "list")
    a <- singleCellTK:::descriptionSoupX()
    testthat::expect_type(a, "list")
})
