app <- ShinyDriver$new("../../", seed = 20)
app$snapshotInit("seuratUI")

app$setInputs(uploadChoice = "example")
app$setInputs(selectExampleData = "fluidigm_pollen")
app$setInputs(addExampleImport = "click")
app$setInputs(uploadData = "click")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(navbar = "Seurat")
app$setInputs(normalize_button = "click")
app$setInputs(scale_button = "click")
app$setInputs(activePanelSelectSeurat = "Highly Variable Genes")
app$setInputs(hvg_method = "dispersion")
app$setInputs(find_hvg_button = "click")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(hvg_method = "mean.var.plot")
app$setInputs(find_hvg_button = "click")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(hvg_method = "vst")
app$setInputs(find_hvg_button = "click")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(activePanelSelectSeurat = "Dimensionality Reduction")
app$setInputs(pca_compute_jackstraw = TRUE)
app$setInputs(pca_no_components = character(0))
app$setInputs(pca_no_components = 2)
app$setInputs(pca_no_components = 20)
app$setInputs(run_pca_button = "click")
app$setInputs(seuratPCAPlotTabset = "PCA Plot")
app$setInputs(picker_dimheatmap_components_pca = character(0))
app$setInputs(plot_heatmap_pca_button = "click")
app$setInputs(slider_dimheatmap_pca = 2)
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(seuratPCAPlotTabset = "Elbow Plot")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(seuratPCAPlotTabset = "JackStraw Plot")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(seuratPCAPlotTabset = "Heatmap Plot")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(activePanelSelectSeurat = "tSNE/UMAP")
app$setInputs(run_tsne_button = "click")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(tsneUmapTabsetSeurat = "UMAP")
app$setInputs(run_umap_button = "click")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(activePanelSelectSeurat = "Clustering")
app$setInputs(find_clusters_button = "click")
app$setInputs(seuratClusteringPlotTabset = "PCA Plot")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(seuratClusteringPlotTabset = "tSNE Plot")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(seuratClusteringPlotTabset = "UMAP Plot")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
app$setInputs(activePanelSelectSeurat = "Find Markers")
app$setInputs(seuratFindMarkerSelectPhenotype = "Biological_Condition")
app$setInputs(seuratFindMarkerRun = "click")
app$setInputs(activePanelSelectSeurat = "Heatmap Plot")
Sys.sleep(1)
app$snapshot(items = list(output = ""), screenshot = TRUE)
